import sympy as sp
import scipy as si
import numpy as np


class Udk:

    def __int__(self):
        pass

    def Constraint_Parameters(self, f, *symbols, **flags):
        r"""
            symbolically solves for A and b matrix for the udwadia kalaba theory."""

        return

    def ideal_Constraint_force(self, m, q, A, b):
        B = A @ si.linalg.fractional_matrix_power(m, -1 / 2)
        a = si.linalg.inv(m) @ q
        Msqrt = si.linalg.fractional_matrix_power(m, 1 / 2)
        e = b - A @ a  # the error vector
        Bplus = np.linalg.pinv(B.astype(np.float64))
        K = Msqrt @ Bplus  # weighted Moore-Penrose generalized inverse of the weighted constraint matrix A
        return K @ e

    def non_ideal_Constraint_force(self,m,A,c):
        B = A @ si.linalg.fractional_matrix_power(m, -1 / 2)
        M_sqrt = si.linalg.fractional_matrix_power(m, 1 / 2)
        M_negsqrt = si.linalg.fractional_matrix_power(m, -1 / 2)
        BplusB = np.linalg.pinv(B.astype(np.float64)) @ B
        I = np.identity(BplusB.shape[0])
        return M_sqrt @ (I - BplusB) @ M_negsqrt @ c



"""Python setup.py for pyUdK package"""
import io
import os
from setuptools import find_packages, setup

VERSION = '0.0.4'


def read(*paths, **kwargs):
    """Read the contents of a text file safely. >>> read("project_name", "VERSION")
    '0.1.0' >>> read("README.md") ..."""
    content = ""
    with io.open(
            os.path.join(os.path.dirname(__file__), *paths),
            encoding=kwargs.get("encoding", "utf8"),
    ) as open_file:
        content = open_file.read().strip()
    return content


def read_requirements(path):
    return [
        line.strip()
        for line in read(path).split("\n")
        if not line.startswith(('"', "#", "-", "git+"))
    ]


setup(
    name="pyUdk",
    version=VERSION,
    description="Computes Udwadia–Kalaba constraint forces of an "
                "equation of motion of a constrained mechanical system.",
    url="https://github.com/Eddy-Morgan/PyUdK",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="Edward Morgan",
    packages=find_packages(exclude=["tests", ".github"]),
    install_requires=read_requirements("requirements.txt"),
    keywords=['python', 'pyUdK', 'Udwadia Kalaba', 'constraint', 'gaussian', 'equation of motion',
              'constrained mechanical system'],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "License :: OSI Approved :: MIT License",
        "Topic :: Scientific/Engineering"
    ]
)
